@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from __future__ import with_statement
 
 import copy
 import cPickle
@@ -8,23 +9,38 @@ except ImportError:
     import simplejson as json
 import os
 import random
-import unittest
+import socket
+import platform
+
+if platform.python_version() < '2.7':
+    unittest = __import__('unittest2')
+else:
+    import unittest
 import uuid
 import time
 
 from riak import RiakClient
-from riak import RiakPbcTransport, RiakPbcCachedTransport
-from riak import RiakHttpTransport, RiakHttpPoolTransport, RiakHttpReuseTransport
+from riak import RiakPbcTransport
+from riak import RiakHttpTransport
 from riak import RiakKeyFilter, key_filter
+from riak.riak_index_entry import RiakIndexEntry
 from riak.mapreduce import RiakLink
 from riak.test_server import TestServer
 
+try:
+    import riak.transports.riakclient_pb2
+    HAVE_PROTO = True
+except ImportError:
+    HAVE_PROTO = False
+
 HOST = os.environ.get('RIAK_TEST_HOST', 'localhost')
 HTTP_HOST = os.environ.get('RIAK_TEST_HTTP_HOST', HOST)
 PB_HOST = os.environ.get('RIAK_TEST_PB_HOST', HOST)
 HTTP_PORT = int(os.environ.get('RIAK_TEST_HTTP_PORT', '8098'))
 PB_PORT = int(os.environ.get('RIAK_TEST_PB_PORT', '8087'))
 SKIP_SEARCH = int(os.environ.get('SKIP_SEARCH', '0'))
+SKIP_LUWAK = int(os.environ.get('SKIP_LUWAK', '0'))
+SKIP_INDEXES = int(os.environ.get('SKIP_INDEXES', '0'))
 USE_TEST_SERVER = int(os.environ.get('USE_TEST_SERVER', '0'))
 
 if USE_TEST_SERVER:
@@ -35,7 +51,6 @@ if USE_TEST_SERVER:
     test_server.prepare()
     test_server.start()
 
-SKIP_LUWAK = int(os.environ.get('SKIP_LUWAK', '0'))
 
 class NotJsonSerializable(object):
 
@@ -75,6 +90,13 @@ class BaseTestCase(object):
     def setUp(self):
         self.client = self.create_client()
 
+        # make sure these are not left over from a previous, failed run
+        bucket = self.client.bucket('bucket')
+        o = bucket.get('nonexistent_key_json')
+        o.delete()
+        o = bucket.get('nonexistent_key_binary')
+        o.delete()
+
     def test_is_alive(self):
         self.assertTrue(self.client.is_alive())
 
@@ -89,12 +111,33 @@ class BaseTestCase(object):
         self.assertEqual(obj.get_key(), 'foo')
         self.assertEqual(obj.get_data(), rand)
 
-        #unicode input should raise a TypeError,
-        #to avoid issues further down the line
-        self.assertRaises(TypeError, self.client.bucket, u'bucket')
+        # unicode objects are fine, as long as they don't
+        # contain any non-ASCII chars
+        self.client.bucket(u'bucket')
+        self.assertRaises(TypeError, self.client.bucket, u'búcket')
+        self.assertRaises(TypeError, self.client.bucket, 'búcket')
+
+        bucket.get(u'foo')
+        self.assertRaises(TypeError, bucket.get, u'føø')
+        self.assertRaises(TypeError, bucket.get, 'føø')
+
+        self.assertRaises(TypeError, bucket.new, u'foo', 'éå')
         self.assertRaises(TypeError, bucket.new, u'foo', 'éå')
         self.assertRaises(TypeError, bucket.new, 'foo', u'éå')
-        self.assertRaises(TypeError, bucket.get, u'foo')
+        self.assertRaises(TypeError, bucket.new, 'foo', u'éå')
+
+    def test_generate_key(self):
+        # Ensure that Riak generates a random key when
+        # the key passed to bucket.new() is None.
+        bucket = self.client.bucket('random_key_bucket')
+        existing_keys = bucket.get_keys()
+        o = bucket.new(None, data={})
+        self.assertIsNone(o.get_key())
+        o.store()
+        self.assertIsNotNone(o.get_key())
+        self.assertNotIn('/', o.get_key())
+        self.assertNotIn(o.get_key(), existing_keys)
+        self.assertEqual(len(bucket.get_keys()), len(existing_keys) + 1)
 
     def test_binary_store_and_get(self):
         bucket = self.client.bucket('bucket')
@@ -192,16 +235,40 @@ class BaseTestCase(object):
         bucket.set_rw("one")
         self.assertEqual(bucket.get_rw(), "one")
 
-    def test_siblings(self):
-        # Set up the bucket, clear any existing object...
-        bucket = self.client.bucket('multiBucket')
-        bucket.set_allow_multiples(True)
-        obj = bucket.get('foo')
+    def test_primary_quora(self):
+        bucket = self.client.bucket('primary_quora')
+        self.assertEqual(bucket.get_pr(), "default")
+        self.assertEqual(bucket.get_pw(), "default")
+
+        bucket.set_pr(1)
+        self.assertEqual(bucket.get_pr(), 1)
+
+        bucket.set_pw("quorum")
+        self.assertEqual(bucket.get_pw(), "quorum")
+
+    def test_if_none_match(self):
+        bucket = self.client.bucket('if_none_match_test')
+        obj = bucket.get('obj')
         obj.delete()
 
         obj.reload()
         self.assertFalse(obj.exists())
-        self.assertEqual(obj.get_data(), None)
+        obj.set_data(["first store"])
+        obj.store()
+
+        obj.set_data(["second store"])
+        with self.assertRaises(Exception):
+            obj.store(if_none_match=True)
+
+    def test_siblings(self):
+        # Set up the bucket, clear any existing object...
+        bucket = self.client.bucket('multiBucket')
+        bucket.set_allow_multiples(True)
+        obj = bucket.get_binary('foo')
+        # Even if it previously existed, let's store a base resolved version
+        # from which we can diverge by sending a stale vclock.
+        obj.set_data('start')
+        obj.store()
 
         # Store the same object five times...
         vals = set()
@@ -213,9 +280,10 @@ class BaseTestCase(object):
                 if randval not in vals:
                     break
 
-            other_obj = other_bucket.new('foo', randval)
+            other_obj = other_bucket.new_binary('foo', str(randval))
+            other_obj._vclock = obj._vclock
             other_obj.store()
-            vals.add(randval)
+            vals.add(str(randval))
 
         # Make sure the object has itself plus four siblings...
         obj.reload()
@@ -236,9 +304,6 @@ class BaseTestCase(object):
         self.assertEqual(obj.get_sibling_count(), 0)
         self.assertEqual(obj.get_data(), obj3.get_data())
 
-        # Clean up for next test...
-        obj.delete()
-
     def test_javascript_source_map(self):
         # Create the object...
         bucket = self.client.bucket("bucket")
@@ -249,9 +314,16 @@ class BaseTestCase(object):
             "function (v) { return [JSON.parse(v.values[0].data)]; }").run()
         self.assertEqual(result, [2])
 
-        #test unicode function
+        # test ASCII-encodable unicode is accepted
+        mr.map(u"function (v) { return [JSON.parse(v.values[0].data)]; }")
+
+        # test non-ASCII-encodable unicode is rejected
         self.assertRaises(TypeError, mr.map,
-            u"function (v) { return [JSON.parse(v.values[0].data)]; }")
+            u"function (v) { /* æ */ return [JSON.parse(v.values[0].data)]; }")
+
+        # test non-ASCII-encodable string is rejected
+        self.assertRaises(TypeError, mr.map,
+            "function (v) { /* æ */ return [JSON.parse(v.values[0].data)]; }")
 
     def test_javascript_named_map(self):
         # Create the object...
@@ -396,14 +468,48 @@ class BaseTestCase(object):
     def test_store_and_get_links(self):
         # Create the object...
         bucket = self.client.bucket("bucket")
-        bucket.new("foo", 2) \
+        bucket.new_binary("test_store_and_get_links", '2') \
             .add_link(bucket.new("foo1")) \
             .add_link(bucket.new("foo2"), "tag") \
             .add_link(bucket.new("foo3"), "tag2!@#%^&*)") \
             .store()
-        obj = bucket.get("foo")
+        obj = bucket.get("test_store_and_get_links")
         links = obj.get_links()
         self.assertEqual(len(links), 3)
+        for l in links:
+            if (l.get_key() == "foo1"):
+                self.assertEqual(l.get_tag(), "bucket")
+            elif (l.get_key() == "foo2"):
+                self.assertEqual(l.get_tag(), "tag")
+            elif (l.get_key() == "foo3"):
+                self.assertEqual(l.get_tag(), "tag2!@#%^&*)")
+            else:
+                self.assertEqual("unknown key", l.get_key())
+
+    def test_set_links(self):
+        # Create the object
+        bucket = self.client.bucket("bucket")
+        bucket.new("foo", 2).set_links([bucket.new("foo1"),
+            (bucket.new("foo2"), "tag"),
+            RiakLink("bucket", "foo2", "tag2")]).store()
+        obj = bucket.get("foo")
+        links = sorted(obj.get_links(), key=lambda x: x.get_key())
+        self.assertEqual(len(links), 3)
+        self.assertEqual(links[0].get_key(), "foo1")
+        self.assertEqual(links[1].get_key(), "foo2")
+        self.assertEqual(links[1].get_tag(), "tag")
+        self.assertEqual(links[2].get_key(), "foo2")
+        self.assertEqual(links[2].get_tag(), "tag2")
+
+    def test_set_links_all_links(self):
+        bucket = self.client.bucket("bucket")
+        foo1 = bucket.new("foo", 1)
+        foo2 = bucket.new("foo2", 2).store()
+        links = [RiakLink("bucket", "foo2")]
+        foo1.set_links(links, True)
+        links = foo1.get_links()
+        self.assertEqual(len(links), 1)
+        self.assertEqual(links[0].get_key(), "foo2")
 
     def test_link_walking(self):
         # Create the object...
@@ -438,10 +544,8 @@ class BaseTestCase(object):
         self.assertEqual(o.get_content_type(), "application/octet-stream")
         o.delete()
 
-
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_search_integration(self):
-        if SKIP_SEARCH:
-            return True
         # Create some objects to search across...
         bucket = self.client.bucket("searchbucket")
         bucket.new("one", {"foo":"one", "bar":"red"}).store()
@@ -463,7 +567,8 @@ class BaseTestCase(object):
     def test_store_binary_object_from_file(self):
         bucket = self.client.bucket('bucket')
         rand = str(self.randint())
-        obj = bucket.new_binary_from_file('foo_from_file', os.path.dirname(__file__) + "/test_all.py")
+        filepath = os.path.join(os.path.dirname(__file__), 'test_all.py')
+        obj = bucket.new_binary_from_file('foo_from_file', filepath)
         obj.store()
         obj = bucket.get_binary('foo_from_file')
         self.assertNotEqual(obj.get_data(), None)
@@ -472,7 +577,9 @@ class BaseTestCase(object):
     def test_store_binary_object_from_file_should_use_default_mimetype(self):
         bucket = self.client.bucket('bucket')
         rand = str(self.randint())
-        obj = bucket.new_binary_from_file('foo_from_file', os.path.dirname(__file__) + '/../../THANKS')
+        filepath = os.path.join(os.path.dirname(os.path.abspath(__file__)),
+                                os.pardir, os.pardir, 'THANKS')
+        obj = bucket.new_binary_from_file('foo_from_file', filepath)
         obj.store()
         obj = bucket.get_binary('foo_from_file')
         self.assertEqual(obj.get_content_type(), 'application/octet-stream')
@@ -499,6 +606,216 @@ class BaseTestCase(object):
         buckets = self.client.get_buckets()
         self.assertTrue("list_bucket" in buckets)
 
+    def is_2i_supported(self):
+        # Immediate test to see if 2i is even supported w/ the backend
+        try:
+            self.client.index('foo','bar_bin','baz').run()
+            return True
+        except Exception as e:
+            if "indexes_not_supported" in str(e):
+                return False
+            return True # it failed, but is supported!
+
+    @unittest.skipIf(SKIP_INDEXES, 'SKIP_INDEXES is defined')
+    def test_secondary_index_store(self):
+        if not self.is_2i_supported():
+            return True
+
+        # Create a new object with indexes...
+        bucket = self.client.bucket('indexbucket')
+        rand = self.randint()
+        obj = bucket.new('mykey1', rand)
+        obj.add_index('field1_bin', 'val1a')
+        obj.add_index('field1_int', 1011)
+        obj.store()
+
+        # Retrieve the object, check that the correct indexes exist...
+        obj = bucket.get('mykey1')
+        self.assertEqual(['val1a'], sorted(obj.get_indexes('field1_bin')))
+        self.assertEqual(['1011'], sorted(obj.get_indexes('field1_int')))
+
+        # Add more indexes and save...
+        obj.add_index('field1_bin', 'val1b')
+        obj.add_index('field1_int', 1012)
+        obj.store()
+
+        # Retrieve the object, check that the correct indexes exist...
+        obj = bucket.get('mykey1')
+        self.assertEqual(['val1a', 'val1b'], sorted(obj.get_indexes('field1_bin')))
+        self.assertEqual(['1011', '1012'], sorted(obj.get_indexes('field1_int')))
+
+        # Check the get_indexes() function...
+        self.assertEqual([
+                RiakIndexEntry('field1_bin', 'val1a'),
+                RiakIndexEntry('field1_bin', 'val1b'),
+                RiakIndexEntry('field1_int', 1011),
+                RiakIndexEntry('field1_int', 1012)
+                ], sorted(obj.get_indexes()))
+
+        # Delete an index...
+        obj.remove_index('field1_bin', 'val1a')
+        obj.remove_index('field1_int', 1011)
+        obj.store()
+
+        # Retrieve the object, check that the correct indexes exist...
+        obj = bucket.get('mykey1')
+        self.assertEqual(['val1b'], sorted(obj.get_indexes('field1_bin')))
+        self.assertEqual(['1012'], sorted(obj.get_indexes('field1_int')))
+
+        # Check duplicate entries...
+        obj.add_index('field1_bin', 'val1a')
+        obj.add_index('field1_bin', 'val1a')
+        obj.add_index('field1_bin', 'val1a')
+        obj.add_index('field1_int', 1011)
+        obj.add_index('field1_int', 1011)
+        obj.add_index('field1_int', 1011)
+
+        self.assertEqual([
+                RiakIndexEntry('field1_bin', 'val1a'),
+                RiakIndexEntry('field1_bin', 'val1b'),
+                RiakIndexEntry('field1_int', 1011),
+                RiakIndexEntry('field1_int', 1012)
+                ], sorted(obj.get_indexes()))
+
+        obj.store()
+        obj = bucket.get('mykey1')
+
+        self.assertEqual([
+                RiakIndexEntry('field1_bin', 'val1a'),
+                RiakIndexEntry('field1_bin', 'val1b'),
+                RiakIndexEntry('field1_int', 1011),
+                RiakIndexEntry('field1_int', 1012)
+                ], sorted(obj.get_indexes()))
+
+        # Clean up...
+        bucket.get('mykey1').delete()
+
+    @unittest.skipIf(SKIP_INDEXES, 'SKIP_INDEXES is defined')
+    def test_set_indexes(self):
+        if not self.is_2i_supported():
+            return True
+
+        bucket = self.client.bucket('indexbucket')
+        foo = bucket.new('foo', 1)
+        foo.set_indexes((('field1_bin', 'test'), ('field2_int', 1337))).store()
+        result = self.client.index('indexbucket', 'field2_int', 1337).run()
+        self.assertEqual(1, len(result))
+        self.assertEqual('foo', result[0].get_key())
+
+        result = self.client.index('indexbucket', 'field1_bin', 'test').run()
+        self.assertEqual(1, len(result))
+        self.assertEqual('foo', result[0].get_key())
+
+    @unittest.skipIf(SKIP_INDEXES, 'SKIP_INDEXES is defined')
+    def test_remove_indexes(self):
+        if not self.is_2i_supported():
+            return True
+
+        bucket = self.client.bucket('indexbucket')
+        bar = bucket.new('bar', 1).add_index('bar_int', 1).add_index('bar_int', 2).add_index('baz_bin', 'baz').store()
+        result = self.client.index('indexbucket', 'bar_int', 1).run()
+        self.assertEqual(1, len(result))
+        self.assertEqual(3, len(bar.get_indexes()))
+        self.assertEqual(2, len(bar.get_indexes('bar_int')))
+        
+        # remove all indexes
+        bar = bar.remove_indexes().store()
+        result = self.client.index('indexbucket', 'bar_int', 1).run()
+        self.assertEqual(0, len(result))
+        result = self.client.index('indexbucket', 'baz_bin', 'baz').run()
+        self.assertEqual(0, len(result))
+        self.assertEqual(0, len(bar.get_indexes()))
+        self.assertEqual(0, len(bar.get_indexes('bar_int')))
+        self.assertEqual(0, len(bar.get_indexes('baz_bin')))
+
+        # add index again
+        bar = bar.add_index('bar_int', 1).add_index('bar_int', 2).add_index('baz_bin', 'baz').store()
+        # remove all index with field='bar_int'
+        bar = bar.remove_index(field='bar_int').store()
+        result = self.client.index('indexbucket', 'bar_int', 1).run()
+        self.assertEqual(0, len(result))
+        result = self.client.index('indexbucket', 'bar_int', 2).run()
+        self.assertEqual(0, len(result))
+        result = self.client.index('indexbucket', 'baz_bin', 'baz').run()
+        self.assertEqual(1, len(result))
+        self.assertEqual(1, len(bar.get_indexes()))
+        self.assertEqual(0, len(bar.get_indexes('bar_int')))
+        self.assertEqual(1, len(bar.get_indexes('baz_bin')))
+
+        # add index again
+        bar = bar.add_index('bar_int', 1).add_index('bar_int', 2).add_index('baz_bin', 'baz').store()
+        # remove an index field value pair
+        bar = bar.remove_index(field='bar_int', value=2).store()
+        result = self.client.index('indexbucket', 'bar_int', 1).run()
+        self.assertEqual(1, len(result))
+        result = self.client.index('indexbucket', 'bar_int', 2).run()
+        self.assertEqual(0, len(result))
+        result = self.client.index('indexbucket', 'baz_bin', 'baz').run()
+        self.assertEqual(1, len(result))
+        self.assertEqual(2, len(bar.get_indexes()))
+        self.assertEqual(1, len(bar.get_indexes('bar_int')))
+        self.assertEqual(1, len(bar.get_indexes('baz_bin')))
+
+    @unittest.skipIf(SKIP_INDEXES, 'SKIP_INDEXES is defined')
+    def test_secondary_index_query(self):
+        if not self.is_2i_supported():
+            return True
+
+        bucket = self.client.bucket('indexbucket')
+
+        bucket.\
+            new('mykey1', 'data1').\
+            add_index('field1_bin', 'val1').\
+            add_index('field2_int', 1001).\
+            store()
+        bucket.\
+            new('mykey2', 'data1').\
+            add_index('field1_bin', 'val2').\
+            add_index('field2_int', 1002).\
+            store()
+        bucket.\
+            new('mykey3', 'data1').\
+            add_index('field1_bin', 'val3').\
+            add_index('field2_int', 1003).\
+            store()
+        bucket.\
+            new('mykey4', 'data1').\
+            add_index('field1_bin', 'val4').\
+            add_index('field2_int', 1004).\
+            store()
+
+        # Test an equality query...
+        results = self.client.index('indexbucket', 'field1_bin', 'val2').run()
+        self.assertEquals(1, len(results))
+        self.assertEquals('mykey2', results[0].get_key())
+
+        # Test a range query...
+        results = self.client.index('indexbucket', 'field1_bin', 'val2', 'val4').run()
+        vals = set()
+        for i in results:
+            vals.add(i.get_key())
+        self.assertEquals(3, len(results))
+        self.assertEquals(set(['mykey2', 'mykey3', 'mykey4']), vals)
+
+        # Test an equality query...
+        results = self.client.index('indexbucket', 'field2_int', 1002).run()
+        self.assertEquals(1, len(results))
+        self.assertEquals('mykey2', results[0].get_key())
+
+        # Test a range query...
+        results = self.client.index('indexbucket', 'field2_int', 1002, 1004).run()
+        vals = set()
+        for i in results:
+            vals.add(i.get_key())
+        self.assertEquals(3, len(results))
+        self.assertEquals(set(['mykey2', 'mykey3', 'mykey4']), vals)
+
+        # Clean up...
+        bucket.get('mykey1').delete()
+        bucket.get('mykey2').delete()
+        bucket.get('mykey3').delete()
+        bucket.get('mykey4').delete()
+
 class MapReduceAliasTestMixIn(object):
     """This tests the map reduce aliases"""
 
@@ -693,6 +1010,8 @@ class RiakPbcTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn,
                                unittest.TestCase):
 
     def setUp(self):
+        if not HAVE_PROTO:
+          self.skipTest('protobuf is unavailable')
         self.host = PB_HOST
         self.port = PB_PORT
         self.transport_class = RiakPbcTransport
@@ -707,22 +1026,54 @@ class RiakPbcTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn,
                             client_id = zero_client_id)
         self.assertEqual(zero_client_id, c.get_client_id()) #
 
-class RiakPbcCachedTransportCase(BaseTestCase, MapReduceAliasTestMixIn,
-                               unittest.TestCase):
-    def setUp(self):
-        self.host = PB_HOST
-        self.port = PB_PORT
-        self.transport_class = RiakPbcCachedTransport
-        super(RiakPbcCachedTransportCase, self).setUp()
+    def test_close_underlying_socket_fails(self):
+        c = RiakClient(PB_HOST, PB_PORT, transport_class = RiakPbcTransport)
 
-    def test_uses_client_id_if_given(self):
-        self.host = PB_HOST
-        self.port = PB_PORT
-        zero_client_id = "\0\0\0\0"
-        c = RiakClient(PB_HOST, PB_PORT,
-                            transport_class = RiakPbcCachedTransport,
-                            client_id = zero_client_id)
-        self.assertEqual(zero_client_id, c.get_client_id()) #
+        bucket = c.bucket('bucket_test_close')
+        rand = self.randint()
+        obj = bucket.new('foo', rand)
+        obj.store()
+        obj = bucket.get('foo')
+        self.assertTrue(obj.exists())
+        self.assertEqual(obj.get_bucket().get_name(), 'bucket_test_close')
+        self.assertEqual(obj.get_key(), 'foo')
+        self.assertEqual(obj.get_data(), rand)
+
+        # Close the underlying socket. This gets a bit sketchy,
+        # since we are reaching into the internals, but there is
+        # no other way to get at the socket
+        conns = c._cm.conns
+        conns[0].sock.close()
+
+        # This shoud fail with a socket error now
+        self.assertRaises(socket.error, bucket.get, 'foo')
+
+    def test_close_underlying_socket_retry(self):
+        c = RiakClient(PB_HOST, PB_PORT, transport_class=RiakPbcTransport,
+                                         transport_options={"max_attempts": 2})
+
+        bucket = c.bucket('bucket_test_close')
+        rand = self.randint()
+        obj = bucket.new('barbaz', rand)
+        obj.store()
+        obj = bucket.get('barbaz')
+        self.assertTrue(obj.exists())
+        self.assertEqual(obj.get_bucket().get_name(), 'bucket_test_close')
+        self.assertEqual(obj.get_key(), 'barbaz')
+        self.assertEqual(obj.get_data(), rand)
+
+        # Close the underlying socket. This gets a bit sketchy,
+        # since we are reaching into the internals, but there is
+        # no other way to get at the socket
+        conns = c._cm.conns
+        conns[0].sock.close()
+
+        # This should work, since we have a retry
+        obj = bucket.get('barbaz')
+        self.assertTrue(obj.exists())
+        self.assertEqual(obj.get_bucket().get_name(), 'bucket_test_close')
+        self.assertEqual(obj.get_key(), 'barbaz')
+        self.assertEqual(obj.get_data(), rand)
 
 
 class RiakHttpTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.TestCase):
@@ -738,15 +1089,6 @@ class RiakHttpTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.
         o = bucket.new("foo", "bar").store(return_body=False)
         self.assertEqual(o.vclock(), None)
 
-    def test_generate_key(self):
-        # Ensure that Riak generates a random key when
-        # the key passed to bucket.new() is None.
-        bucket = self.client.bucket('random_key_bucket')
-        for key in bucket.get_keys():
-            bucket.get(str(key)).delete()
-        bucket.new(None, data={}).store()
-        self.assertEqual(len(bucket.get_keys()), 1)
-
     def test_too_many_link_headers_shouldnt_break_http(self):
         bucket = self.client.bucket("bucket")
         o = bucket.new("lots_of_links", "My god, it's full of links!")
@@ -774,22 +1116,18 @@ class RiakHttpTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.
         bucket.disable_search()
         self.assertFalse(self.client.bucket("no_search_bucket").search_enabled())
 
+    @unittest.skipIf(SKIP_LUWAK, 'SKIP_LUWAK is defined')
     def test_store_file_with_luwak(self):
-        if SKIP_LUWAK:
-            return True
-
-        file = os.path.dirname(__file__) + "/test_all.py"
+        file = os.path.join(os.path.dirname(__file__), "test_all.py")
         with open(file, "r") as input_file:
             data = input_file.read()
 
         key = uuid.uuid1().hex
         self.client.store_file(key, data)
 
+    @unittest.skipIf(SKIP_LUWAK, 'SKIP_LUWAK is defined')
     def test_store_get_file_with_luwak(self):
-        if SKIP_LUWAK:
-            return True
-
-        file = os.path.dirname(__file__) + "/test_all.py"
+        file = os.path.join(os.path.dirname(__file__), "test_all.py")
         with open(file, "r") as input_file:
             data = input_file.read()
 
@@ -799,11 +1137,9 @@ class RiakHttpTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.
         file = self.client.get_file(key)
         self.assertEquals(data, file)
 
+    @unittest.skipIf(SKIP_LUWAK, 'SKIP_LUWAK is defined')
     def test_delete_file_with_luwak(self):
-        if SKIP_LUWAK:
-            return True
-
-        file = os.path.dirname(__file__) + "/test_all.py"
+        file = os.path.join(os.path.dirname(__file__), "test_all.py")
         with open(file, "r") as input_file:
             data = input_file.read()
 
@@ -815,125 +1151,73 @@ class RiakHttpTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.
         file = self.client.get_file(key)
         self.assertIsNone(file)
 
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_solr_search_from_bucket(self):
-        if SKIP_SEARCH:
-            return True
         bucket = self.client.bucket('searchbucket')
         bucket.new("user", {"username": "roidrage"}).store()
         results = bucket.search("username:roidrage")
         self.assertEquals(1, len(results["response"]["docs"]))
 
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_solr_search_with_params_from_bucket(self):
-        if SKIP_SEARCH:
-            return True
         bucket = self.client.bucket('searchbucket')
         bucket.new("user", {"username": "roidrage"}).store()
         results = bucket.search("username:roidrage", wt="xml")
-        self.assertEquals(1, len(list(results.find("result").iter("doc"))))
+        result = results.find("result")
+        if not hasattr(result, "iter"): setattr(result, "iter", result.getiterator)
+        self.assertEquals(1, len(list(result.iter("doc"))))
 
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_solr_search_with_params(self):
-        if SKIP_SEARCH:
-            return True
         bucket = self.client.bucket('searchbucket')
         bucket.new("user", {"username": "roidrage"}).store()
         results = self.client.solr().search("searchbucket", "username:roidrage", wt="xml")
-        self.assertEquals(1, len(list(results.find("result").iter("doc"))))
+        result = results.find("result")
+        if not hasattr(result, "iter"): setattr(result, "iter", result.getiterator)
+        self.assertEquals(1, len(list(result.iter("doc"))))
 
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_solr_search(self):
-        if SKIP_SEARCH:
-            return True
         bucket = self.client.bucket('searchbucket')
         bucket.new("user", {"username": "roidrage"}).store()
         results = self.client.solr().search("searchbucket", "username:roidrage")
         self.assertEquals(1, len(results["response"]["docs"]))
 
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_add_document_to_index(self):
-        if SKIP_SEARCH:
-            return True
-
         self.client.solr().add("searchbucket", {"id": "doc", "username": "tony"})
         results = self.client.solr().search("searchbucket", "username:tony")
         self.assertEquals("tony", results["response"]["docs"][0]["fields"]["username"])
 
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_add_multiple_documents_to_index(self):
-        if SKIP_SEARCH:
-            return True
         self.client.solr().add("searchbucket", {"id": "dizzy", "username": "dizzy"}, {"id": "russell", "username": "russell"})
         results = self.client.solr().search("searchbucket", "username:russell OR username:dizzy")
         self.assertEquals(2, len(results["response"]["docs"]))
 
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_delete_documents_from_search_by_id(self):
-        if SKIP_SEARCH:
-            return True
         self.client.solr().add("searchbucket", {"id": "dizzy", "username": "dizzy"}, {"id": "russell", "username": "russell"})
         self.client.solr().delete("searchbucket", docs=["dizzy"])
         results = self.client.solr().search("searchbucket", "username:russell OR username:dizzy")
         self.assertEquals(1, len(results["response"]["docs"]))
 
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_delete_documents_from_search_by_query(self):
-        if SKIP_SEARCH:
-            return True
         self.client.solr().add("searchbucket", {"id": "dizzy", "username": "dizzy"}, {"id": "russell", "username": "russell"})
         self.client.solr().delete("searchbucket", queries=["username:dizzy", "username:russell"])
         results = self.client.solr().search("searchbucket", "username:russell OR username:dizzy")
         self.assertEquals(0, len(results["response"]["docs"]))
 
+    @unittest.skipIf(SKIP_SEARCH, 'SKIP_SEARCH is defined')
     def test_delete_documents_from_search_by_query_and_id(self):
-        if SKIP_SEARCH:
-            return True
         self.client.solr().add("searchbucket", {"id": "dizzy", "username": "dizzy"}, {"id": "russell", "username": "russell"})
         self.client.solr().delete("searchbucket", docs=["dizzy"], queries=["username:russell"])
         results = self.client.solr().search("searchbucket", "username:russell OR username:dizzy")
         self.assertEquals(0, len(results["response"]["docs"]))
 
-class RiakHttpPoolTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.TestCase):
-
-    def setUp(self):
-        self.host = HTTP_HOST
-        self.port = HTTP_PORT
-        self.transport_class = RiakHttpPoolTransport
-        super(RiakHttpPoolTransportTestCase, self).setUp()
-
-    def test_no_returnbody(self):
-        bucket = self.client.bucket("bucket")
-        o = bucket.new("foo", "bar").store(return_body=False)
-        self.assertEqual(o.vclock(), None)
-
-    def test_generate_key(self):
-        # Ensure that Riak generates a random key when
-        # the key passed to bucket.new() is None.
-        bucket = self.client.bucket('random_key_bucket')
-        for key in bucket.get_keys():
-            bucket.get(str(key)).delete()
-        bucket.new(None, data={}).store()
-        self.assertEqual(len(bucket.get_keys()), 1)
-
-    def test_set_client_id(self):
-        self.client.set_client_id("Client")
-        self.assertEqual(self.client.get_transport().get_client_id(), "Client")
-
-class RiakHttpReuseTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.TestCase):
-
-    def setUp(self):
-        self.host = HTTP_HOST
-        self.port = HTTP_PORT
-        self.transport_class = RiakHttpReuseTransport
-        super(RiakHttpReuseTransportTestCase, self).setUp()
-
-    def test_no_returnbody(self):
-        bucket = self.client.bucket("bucket")
-        o = bucket.new("foo", "bar").store(return_body=False)
-        self.assertEqual(o.vclock(), None)
-
-    def test_generate_key(self):
-        # Ensure that Riak generates a random key when
-        # the key passed to bucket.new() is None.
-        bucket = self.client.bucket('random_key_bucket')
-        for key in bucket.get_keys():
-            bucket.get(str(key)).delete()
-        bucket.new(None, data={}).store()
-        self.assertEqual(len(bucket.get_keys()), 1)
-
+    def test_build_rest_path_excludes_empty_query_params(self):
+        self.assertEquals(self.client.get_transport().build_rest_path(bucket=self.client.bucket("foo"), key="bar", params={'r': None}), "/riak/foo/bar?")
 
 class RiakTestFilter(unittest.TestCase):
     def test_simple(self):
