@@ -17,15 +17,13 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
-import urllib, re
+from __future__ import with_statement
+
+import urllib, re, csv
 from cStringIO import StringIO
-# Use pycurl as first choice, httplib as second choice.
-try:
-    import pycurl
-    HAS_PYCURL = True
-except ImportError:
-    import httplib
-    HAS_PYCURL = False
+import httplib
+import socket
+import errno
 try:
     import json
 except ImportError:
@@ -35,7 +33,10 @@ from transport import RiakTransport
 from riak.metadata import *
 from riak.mapreduce import RiakLink
 from riak import RiakError
+from riak.riak_index_entry import RiakIndexEntry
 from riak.multidict import MultiDict
+from connection import HTTPConnectionManager
+import riak.util
 
 MAX_LINK_HEADER_SIZE = 8192 - 8 # substract length of "Link: " header string and newline
 
@@ -45,9 +46,19 @@ class RiakHttpTransport(RiakTransport) :
     Riak. The Riak API uses HTTP, so there is no persistent
     connection, and the RiakClient object is extremely lightweight.
     """
-    def __init__(self, host='127.0.0.1', port=8098, prefix='riak',
-                 mapred_prefix='mapred',
-                 client_id = None):
+
+    # We're using the new RiakTransport API
+    api = 2
+
+    # The ConnectionManager class that this transport prefers.
+    default_cm = HTTPConnectionManager
+
+    # How many times to retry a request
+    RETRY_COUNT = 3
+
+    def __init__(self, cm,
+                 prefix='riak', mapred_prefix='mapred', client_id=None,
+                 **unused_options):
         """
         Construct a new RiakClient object.
         @param string host - Hostname or IP address (default '127.0.0.1')
@@ -57,8 +68,7 @@ class RiakHttpTransport(RiakTransport) :
         @param string client_id - client id to use for vector clocks
         """
         super(RiakHttpTransport, self).__init__()
-        self._host = host
-        self._port = port
+        self._conns = cm
         self._prefix = prefix
         self._mapred_prefix = mapred_prefix
         self._client_id = client_id
@@ -66,6 +76,9 @@ class RiakHttpTransport(RiakTransport) :
             self._client_id = self.make_random_client_id()
 
     def __copy__(self):
+        ### not implemented right now
+        raise Exception('not implemented')
+        ### we don't have _host and _port. will fix after some refactoring...
         return RiakHttpTransport(self._host, self._port, self._prefix,
                                  self._mapred_prefix)
 
@@ -79,54 +92,41 @@ class RiakHttpTransport(RiakTransport) :
         """
         Check server is alive over HTTP
         """
-        response = self.http_request('GET', self._host, self._port, '/ping')
+        response = self.http_request('GET', '/ping')
         return(response is not None) and (response[1] == 'OK')
 
 
-    def get(self, robj, r, vtag = None) :
+    def get(self, robj, r = None, pr = None, vtag = None) :
         """
         Get a bucket/key from the server
         """
-        params = {'r' : r}
+        params = {'r' : r, 'pr': pr}
         if vtag is not None:
             params['vtag'] = vtag
-        host, port, url = self.build_rest_path(robj.get_bucket(), robj.get_key(),
-                                               params=params)
-        response = self.http_request('GET', host, port, url)
+        url = self.build_rest_path(robj.get_bucket(), robj.get_key(),
+                                   params=params)
+        response = self.http_request('GET', url)
         return self.parse_body(response, [200, 300, 404])
 
-    def put(self, robj, w = None, dw = None, return_body = True):
+    def put(self, robj, w = None, dw = None, pw = None, return_body = True, if_none_match=False):
         """
         Serialize put request and deserialize response
         """
        # Construct the URL...
-        params = {'returnbody' : str(return_body).lower(), 'w' : w, 'dw' : dw}
-        host, port, url = self.build_rest_path(bucket=robj.get_bucket(), key=robj.get_key(),
-                                               params=params)
-
-        # Construct the headers...
-        headers = MultiDict({'Accept' : 'text/plain, */*; q=0.5',
-                             'Content-Type' : robj.get_content_type(),
-                             'X-Riak-ClientId' : self._client_id})
-
-        # Add the vclock if it exists...
-        if robj.vclock() is not None:
-            headers['X-Riak-Vclock'] = robj.vclock()
-
-        # Create the header from metadata
-        links = self.add_links_for_riak_object(robj, headers)
-
-        for key, value in robj.get_usermeta().iteritems():
-            headers['X-Riak-Meta-%s' % key] = value
-
+        params = {'returnbody' : str(return_body).lower(), 'w' : w, 'dw' : dw, 'pw' : pw }
+        url = self.build_rest_path(bucket=robj.get_bucket(), key=robj.get_key(),
+                                   params=params)
+        headers = self.build_put_headers(robj)
+        if if_none_match:
+            headers["If-None-Match"] = "*"
         content = robj.get_encoded_data()
-        return self.do_put(host, port, url, headers, content, return_body, key=robj.get_key())
+        return self.do_put(url, headers, content, return_body, key=robj.get_key())
 
-    def do_put(self, host, port, url, headers, content, return_body=False, key=None):
+    def do_put(self, url, headers, content, return_body=False, key=None):
         if key is None:
-          response = self.http_request('POST', host, port, url, headers, content)
+          response = self.http_request('POST', url, headers, content)
         else:
-          response = self.http_request('PUT', host, port, url, headers, content)
+          response = self.http_request('PUT', url, headers, content)
 
         if return_body:
           return self.parse_body(response, [200, 201, 300])
@@ -134,21 +134,42 @@ class RiakHttpTransport(RiakTransport) :
           self.check_http_code(response, [204])
           return None
 
-    def delete(self, robj, rw):
+    def put_new(self, robj, w=None, dw=None, pw=None, return_body=True, if_none_match=False):
+        """Put a new object into the Riak store, returning its (new) key."""
         # Construct the URL...
-        params = {'rw' : rw}
-        host, port, url = self.build_rest_path(robj.get_bucket(), robj.get_key(),
-                                               params=params)
+        params = {'returnbody' : str(return_body).lower(), 'w' : w, 'dw' : dw, 'pw' : pw}
+        url = self.build_rest_path(bucket=robj.get_bucket(), params=params)
+        headers = self.build_put_headers(robj)
+        if if_none_match:
+            headers["If-None-Match"] = "*"
+        content = robj.get_encoded_data()
+        response = self.http_request('POST', url, headers, content)
+        location = response[0]['location']
+        idx = location.rindex('/')
+        key = location[idx+1:]
+        if return_body:
+            vclock, [(metadata, data)] = self.parse_body(response, [201])
+            return key, vclock, metadata
+        else:
+            self.check_http_code(response, [201])
+            return key, None, None
+
+    def delete(self, robj, rw=None, r = None, w = None, dw = None, pr = None, pw = None):
+        # Construct the URL...
+        params = {'rw' : rw, 'r': r, 'w': w, 'dw': dw, 'pr': pr, 'pw': pw}
+        url = self.build_rest_path(robj.get_bucket(), robj.get_key(),
+                                   params=params)
+        # TODO: Send vclock of robj if it exists
         # Run the operation..
-        response = self.http_request('DELETE', host, port, url)
+        response = self.http_request('DELETE', url)
         self.check_http_code(response, [204, 404])
         return self
 
 
     def get_keys(self, bucket):
         params = {'props' : 'True', 'keys' : 'true'}
-        host, port, url = self.build_rest_path(bucket, params=params)
-        response = self.http_request('GET', host, port, url)
+        url = self.build_rest_path(bucket, params=params)
+        response = self.http_request('GET', url)
 
         headers, encoded_props = response[0:2]
         if headers['http_code'] == 200:
@@ -159,8 +180,8 @@ class RiakHttpTransport(RiakTransport) :
 
     def get_buckets(self):
         params = {'buckets': 'true'}
-        host, port, url = self.build_rest_path(None, params=params)
-        response = self.http_request('GET', host, port, url)
+        url = self.build_rest_path(None, params=params)
+        response = self.http_request('GET', url)
 
         headers, encoded_props = response[0:2]
         if headers['http_code'] == 200:
@@ -172,8 +193,8 @@ class RiakHttpTransport(RiakTransport) :
     def get_bucket_props(self, bucket):
         # Run the request...
         params = {'props' : 'True', 'keys' : 'False'}
-        host, port, url = self.build_rest_path(bucket, params=params)
-        response = self.http_request('GET', host, port, url)
+        url = self.build_rest_path(bucket, params=params)
+        response = self.http_request('GET', url)
 
         headers = response[0]
         encoded_props = response[1]
@@ -188,12 +209,12 @@ class RiakHttpTransport(RiakTransport) :
         """
         Set the properties on the bucket object given
         """
-        host, port, url = self.build_rest_path(bucket)
+        url = self.build_rest_path(bucket)
         headers = {'Content-Type' : 'application/json'}
         content = json.dumps({'props' : props})
 
         # Run the request...
-        response = self.http_request('PUT', host, port, url, headers, content)
+        response = self.http_request('PUT', url, headers, content)
 
         # Handle the response...
         if response is None:
@@ -214,17 +235,21 @@ class RiakHttpTransport(RiakTransport) :
         content = json.dumps(job)
 
         # Do the request...
-        host = self._host
-        port = self._port
         url = "/" + self._mapred_prefix
-        response = self.http_request('POST', host, port, url, {}, content)
+        response = self.http_request('POST', url, {}, content)
+
+        # Make sure the expected status code came back...
+        status = response[0]['http_code']
+        if status != 200:
+            raise Exception('Error running MapReduce operation. Status: ' + str(status) + ' : ' + response[1])
+
         result = json.loads(response[1])
         return result
 
     def check_http_code(self, response, expected_statuses):
         status = response[0]['http_code']
         if not status in expected_statuses:
-            m = 'Expected status ' + str(expected_statuses) + ', received ' + str(status)
+            m = 'Expected status ' + str(expected_statuses) + ', received ' + str(status) + ' : ' + response[1]
             raise Exception(m)
 
     def parse_body(self, response, expected_statuses):
@@ -248,12 +273,8 @@ class RiakHttpTransport(RiakTransport) :
 
         # Check if the server is down(status==0)
         if not status:
-            m = 'Could not contact Riak Server: http://' + self._host + ':' + str(self._port) + '!'
-            raise RiakError(m)
-
-        # Verify that we got one of the expected statuses. Otherwise, raise an exception.
-        if not status in expected_statuses:
-            m = 'Expected status ' + str(expected_statuses) + ', received ' + str(status)
+            ### we need the host/port that was used.
+            m = 'Could not contact Riak Server: http://$HOST:$PORT !'
             raise RiakError(m)
 
         # If 404(Not Found), then clear the object.
@@ -269,7 +290,7 @@ class RiakHttpTransport(RiakTransport) :
 
         # Parse the headers...
         vclock = None
-        metadata = {MD_USERMETA: {}}
+        metadata = {MD_USERMETA: {}, MD_INDEX: []}
         links = []
         for header, value in headers.iteritems():
             if header == 'content-type':
@@ -286,6 +307,14 @@ class RiakHttpTransport(RiakTransport) :
                 metadata[MD_LASTMOD] = value
             elif header.startswith('x-riak-meta-'):
                 metadata[MD_USERMETA][header.replace('x-riak-meta-', '')] = value
+            elif header.startswith('x-riak-index-'):
+                field = header.replace('x-riak-index-', '')
+                reader = csv.reader([value], skipinitialspace=True)
+                for line in reader:
+                    for token in line:
+                        rie = RiakIndexEntry(field, token)
+                        metadata[MD_INDEX].append(rie)
+
             elif header == 'x-riak-vclock':
                 vclock = value
         if links:
@@ -312,9 +341,12 @@ class RiakHttpTransport(RiakTransport) :
         """
         for linkHeader in linkHeaders.strip().split(','):
             linkHeader = linkHeader.strip()
-            matches = re.match("</([^/]+)/([^/]+)/([^/]+)>; ?riaktag=\"([^\']+)\"", linkHeader)
+            matches = re.match("</([^/]+)/([^/]+)/([^/]+)>; ?riaktag=\"([^\']+)\"", linkHeader) or \
+                re.match("</(buckets)/([^/]+)/keys/([^/]+)>; ?riaktag=\"([^\']+)\"", linkHeader)
             if matches is not None:
-                link = RiakLink(matches.group(2), matches.group(3), matches.group(4))
+                link = RiakLink(urllib.unquote_plus(matches.group(2)),
+                                urllib.unquote_plus(matches.group(3)),
+                                urllib.unquote_plus(matches.group(4)))
                 links.append(link)
         return self
 
@@ -325,7 +357,7 @@ class RiakHttpTransport(RiakTransport) :
             for link in links:
                 header = self.to_link_header(link)
                 if len(current_header + header) > MAX_LINK_HEADER_SIZE:
-                    headers.setdefault('Link', []).append(current_header)
+                    headers.add('Link', current_header)
                     current_header = ''
 
                 if current_header != '': header = ', ' + header
@@ -336,19 +368,19 @@ class RiakHttpTransport(RiakTransport) :
         return headers
 
     def get_request(self, uri=None, params=None):
-        host, port, url = self.build_rest_path(bucket=None, params=params, prefix=uri)
-        return self.http_request('GET', host, port, url)
+        url = self.build_rest_path(bucket=None, params=params, prefix=uri)
+        return self.http_request('GET', url)
 
     def store_file(self, key, content_type="application/octet-stream", content=None):
-        host, port, url = self.build_rest_path(prefix='luwak', key=key)
+        url = self.build_rest_path(prefix='luwak', key=key)
         headers = {'Content-Type' : content_type,
                    'X-Riak-ClientId' : self._client_id}
 
-        return self.do_put(host, port, url, headers, content, key=key)
+        return self.do_put(url, headers, content, key=key)
 
     def get_file(self, key):
-        host, port, url = self.build_rest_path(prefix='luwak', key=key)
-        response = self.http_request('GET', host, port, url)
+        url = self.build_rest_path(prefix='luwak', key=key)
+        response = self.http_request('GET', url)
         result = self.parse_body(response, [200, 300, 404])
         if result is not None:
             (vclock, data) = result
@@ -356,23 +388,16 @@ class RiakHttpTransport(RiakTransport) :
             return body
 
     def delete_file(self, key):
-        host, port, url = self.build_rest_path(prefix='luwak', key=key)
-        response = self.http_request('DELETE', host, port, url)
+        url = self.build_rest_path(prefix='luwak', key=key)
+        response = self.http_request('DELETE', url)
         self.parse_body(response, [204, 404])
 
     def post_request(self, uri=None, body=None, params=None, content_type="application/json"):
-        host, port, uri = self.build_rest_path(prefix=uri, params=params)
-        return self.http_request('POST', self._host, self._port, uri, {'Content-Type': content_type}, body) 
+        uri = self.build_rest_path(prefix=uri, params=params)
+        return self.http_request('POST', uri, {'Content-Type': content_type}, body)
 
     # Utility functions used by Riak library.
 
-    @classmethod
-    def get_value(cls, key, array, defaultValue) :
-        if key in array:
-            return array[key]
-        else:
-            return defaultValue
-
     def build_rest_path(self, bucket=None, key=None, params=None, prefix=None) :
         """
         Given a RiakClient, RiakBucket, Key, LinkSpec, and Params,
@@ -394,101 +419,84 @@ class RiakHttpTransport(RiakTransport) :
         if params is not None:
             s = ''
             for key in params.keys():
-                if s != '': s += '&'
-                s += urllib.quote_plus(key) + '=' + urllib.quote_plus(str(params[key]))
+                if params[key] is not None:
+                    if s != '': s += '&'
+                    s += urllib.quote_plus(key) + '=' + urllib.quote_plus(str(params[key]))
             path += '?' + s
 
         # Return.
-        return self._host, self._port, path
+        return path
 
-    @classmethod
-    def http_request(cls, method, host, port, url, headers = None, obj = '') :
+    def build_put_headers(self, robj):
+        """Build the headers for a POST/PUT request."""
+
+        # Construct the headers...
+        headers = MultiDict({'Accept' : 'text/plain, */*; q=0.5',
+                             'Content-Type' : robj.get_content_type(),
+                             'X-Riak-ClientId' : self._client_id})
+
+        # Add the vclock if it exists...
+        if robj.vclock() is not None:
+            headers['X-Riak-Vclock'] = robj.vclock()
+
+        # Create the header from metadata
+        links = self.add_links_for_riak_object(robj, headers)
+
+        for key, value in robj.get_usermeta().iteritems():
+            headers['X-Riak-Meta-%s' % key] = value
+
+        for rie in robj.get_indexes():
+            key = 'X-Riak-Index-%s' % rie.get_field()
+            if key in headers:
+                headers[key] += ", " + rie.get_value()
+            else:
+                headers[key] = rie.get_value()
+
+        return headers
+
+    def http_request(self, method, uri, headers=None, body='') :
         """
         Given a Method, URL, Headers, and Body, perform and HTTP request,
-        and return an array of arity 2 containing an associative array of
-        response headers and the response body.
+        and return a 2-tuple containing a dictionary of response headers
+        and the response body.
         """
-        if not headers:
-            headers = {}
-        if HAS_PYCURL:
-            return cls.pycurl_request(method, host, port, url, headers, obj)
-        else:
-            return cls.httplib_request(method, host, port, url, headers, obj)
-
-
-    @classmethod
-    def httplib_request(cls, method, host, port, uri, headers = None, body=''):
-        if not headers:
+        if headers is None:
             headers = {}
         # Run the request...
-        client = None
-        response = None
-        try:
-            client = httplib.HTTPConnection(host, port)
-            client.request(method, uri, body, headers)
-            response = client.getresponse()
+        for retry in range(self.RETRY_COUNT):
+            with self._conns.withconn() as conn:
+                ### should probably build this try/except into a custom
+                ### contextmanager for the connection.
+                try:
+                    conn.request(method, uri, body, headers)
+                    response = conn.getresponse()
 
-            # Get the response headers...
-            response_headers = {'http_code': response.status}
-            for (key, value) in response.getheaders():
-                response_headers[key.lower()] = value
+                    try:
+                        # Get the response headers...
+                        response_headers = {'http_code': response.status}
+                        for (key, value) in response.getheaders():
+                            response_headers[key.lower()] = value
 
-            # Get the body...
-            response_body = response.read()
-            response.close()
+                        # Get the body...
+                        response_body = response.read()
+                    finally:
+                      response.close()
 
-            return response_headers, response_body
-        except:
-            if client is not None: client.close()
-            if response is not None: response.close()
-            raise
+                    return response_headers, response_body
+                except socket.error, e:
+                    conn.close()
+                    if e[0] == errno.ECONNRESET:
+                        # Grab another connection and try again.
+                        continue
+                    # Don't know how to handle this.
+                    raise
+                except httplib.HTTPException:
+                    # Just close the connection and try again.
+                    conn.close()
+                    continue
 
-
-    @classmethod
-    def pycurl_request(cls, method, host, port, uri, headers, body=''):
-        if not headers:
-            headers = {}
-        url = "http://" + host + ":" + str(port) + uri
-        # Set up Curl...
-        client = pycurl.Curl()
-        client.setopt(pycurl.URL, url)
-        client.setopt(pycurl.HTTPHEADER, cls.build_headers(headers))
-        if method == 'GET':
-            client.setopt(pycurl.HTTPGET, 1)
-        elif method == 'POST':
-            client.setopt(pycurl.POST, 1)
-            client.setopt(pycurl.POSTFIELDS, body)
-        elif method == 'PUT':
-            client.setopt(pycurl.CUSTOMREQUEST, method)
-            client.setopt(pycurl.POSTFIELDS, body)
-        elif method == 'DELETE':
-            client.setopt(pycurl.CUSTOMREQUEST, method)
-
-        # Capture the response headers...
-        response_headers_io = StringIO()
-        client.setopt(pycurl.HEADERFUNCTION, response_headers_io.write)
-
-        # Capture the response body...
-        response_body_io = StringIO()
-        client.setopt(pycurl.WRITEFUNCTION, response_body_io.write)
-
-        try:
-            # Run the request.
-            client.perform()
-            http_code = client.getinfo(pycurl.HTTP_CODE)
-            client.close()
-
-            # Get the headers...
-            response_headers = cls.parse_http_headers(response_headers_io.getvalue())
-            response_headers['http_code'] = http_code
-
-            # Get the body...
-            response_body = response_body_io.getvalue()
-
-            return response_headers, response_body
-        except:
-            if client is not None: client.close()
-            raise
+        # No luck, even with retrying.
+        raise RiakError("could not get a response")
 
     @classmethod
     def build_headers(cls, headers):
@@ -515,101 +523,3 @@ class RiakHttpTransport(RiakTransport) :
             else:
                 retVal[key] = value
         return retVal
-
-import socket
-
-class RiakHttpReuseTransport(RiakHttpTransport):
-    """
-    Reuse sockets
-    """
-
-    def __init__(self, host='127.0.0.1', port=8098, prefix='riak',
-                 mapred_prefix='mapred',
-                 client_id=None):
-        super(RiakHttpReuseTransport, self).__init__(host=host,
-                                                     port=port,
-                                                     prefix=prefix,
-                                                     mapred_prefix=
-                                                     mapred_prefix,
-                                                     client_id=client_id)
-
-    def __copy__(self):
-        return RiakHttpReuseTransport(self._host, self._port, self._prefix,
-                                      self._mapred_prefix)
-
-    @classmethod
-    def httplib_request(cls, method, host, port, uri, headers, body=''):
-        # Run the request...
-        client = None
-        response = None
-        try:
-            client = httplib.HTTPConnection(host, port)
-
-            #handle the connection myself, try to reuse sockets
-            client.auto_open = 0
-            client.connect()
-            client.sock.setsockopt(
-                socket.SOL_SOCKET, socket.SO_REUSEADDR,
-                client.sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR) | 1)
-
-            client.request(method, uri, body, headers)
-            response = client.getresponse()
-
-            # Get the response headers...
-            response_headers = {'http_code': response.status}
-            for (key, value) in response.getheaders():
-                response_headers[key.lower()] = value
-
-            # Get the body...
-            response_body = response.read()
-            response.close()
-
-            #close, this does not make any difference
-            client.close()
-
-            return response_headers, response_body
-        except:
-            if client is not None: client.close()
-            if response is not None: response.close()
-            raise
-
-import urllib3
-
-class RiakHttpPoolTransport(RiakHttpTransport):
-    """
-    Use HTTP pool
-    """
-
-    http_pool = None
-
-    def __init__(self, host='127.0.0.1', port=8098, prefix='riak',
-                 mapred_prefix='mapred',
-                 client_id=None):
-        super(RiakHttpPoolTransport, self).__init__(host=host,
-                                                    port=port,
-                                                    prefix=prefix,
-                                                    mapred_prefix=
-                                                    mapred_prefix,
-                                                    client_id=client_id)
-
-    def __copy__(self):
-        return RiakHttpPoolTransport(self._host, self._port, self._prefix,
-                                     self._mapred_prefix)
-
-    @classmethod
-    def httplib_request(cls, method, host, port, uri, headers, body=''):
-        try:
-            if cls.http_pool is None:
-                cls.http_pool = urllib3.connection_from_url('http://%s:%d' % (host, port), maxsize=10)
-
-            response = cls.http_pool.urlopen(method, uri, body, headers)
-
-            response_headers = {'http_code': response.status}
-            for key, value in response.getheaders().iteritems():
-                response_headers[key.lower()] = value
-
-            response_body = response.data
-
-            return response_headers, response_body
-        except:
-            raise
